//******************************************************************
//* TELEGESIS ETRX2 DRIVER
//* LAST UPDATED 07/31/2008
//* FIRMWARE BY FRED EADY FOR SERVO MAGAZINE
//* CHANGES/ADDITIONS
//******************************************************************
#include <pic18.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <htc.h>
#include <stdlib.h>

// Settings for PIC18F2620
__CONFIG(1,IESODIS & FCMDIS & RCIO);
__CONFIG(2,BORDIS & PWRTDIS & WDTDIS);
__CONFIG(3,MCLREN & LPT1DIS & PBDIGITAL);
__CONFIG(4,DEBUGEN & LVPDIS & XINSTDIS & STVREN);
__CONFIG(5,UNPROTECT);
__CONFIG(6,WRTEN);
__IDLOC(0731);

//******************************************************************
//*	FUNCTION PROTOTYPES
//******************************************************************
void interrupt EUSART(void);
void init(void);
void putch(char c);
unsigned int recvchar(void);
unsigned int sendchar(unsigned int data);
char CharInQueue(void);
void initEUSART(void);
void cleaner(void);
//******************************************************************
//*	PIC18F2620 RAM Definitions
//******************************************************************
unsigned int temp16;
char usart_data,tmphead,tmptail;
char our_buffer[32];
//******************************************************************
//    MACROS AND PORT/PIN DEFINITIONS
//******************************************************************
#define bitset(var, bitno) ((var) |= 1 << (bitno))
#define bitclr(var, bitno) ((var) &= ~(1 << (bitno)))
#define make8(var,offset)	((unsigned int)var >> (offset * 8)) & 0x00FF
#define	make16(varhigh,varlow)	(((unsigned int)varhigh & 0xFF)* 0x100) + ((unsigned int)varlow & 0x00FF)
#define make32(var1,var2,var3,var4) \
		((unsigned long)var1<<24)+((unsigned long)var2<<16)+ \
		((unsigned long)var3<<8)+((unsigned long)var4)		
#define make32i(var1,var2) ((unsigned long)var1<<16)+((unsigned long)var2)
//******************************************************************
//*	GENERAL DEFINITONS
//******************************************************************
#define	act_led	LATA5
//******************************************************************
//*	INTERRUPT DEFINITIONS
//******************************************************************
#define		disable_RXint	  	RCIE = 0
#define		enable_RXint	  	RCIE = 1
#define		disable_TMR2int  	TMR2IE = 0
#define		enable_TMR2int	 	TMR2IE = 1
#define		disable_TMR3int  	TMR3IE = 0
#define		enable_TMR3int	 	TMR3IE = 1
#define		disable_TMR1int  	TMR1IE = 0
#define		enable_TMR1int	 	TMR1IE = 1
#define		disable_TMR0int  	TMR0IE = 0
#define		enable_TMR0int	 	TMR0IE = 1

#define		enable_GLOBALint  	IPEN = 0;				\
							  	GIE = 1;				\
							  	PEIE = 1;
#define		disable_GLOBALint 	IPEN = 0;				\
							  	GIE = 0;				\
							  	PEIE = 0;
#define		TIMER0ON			TMR0ON = 1
#define		TIMER0OFF			TMR0ON = 0
#define		TIMER1ON			TMR1ON = 1
#define		TIMER1OFF			TMR1ON = 0
#define		TIMER2ON		  	TMR2ON = 1
#define		TIMER2OFF		  	TMR2ON = 0
#define		TIMER3ON		  	TMR3ON = 1
#define		TIMER3OFF		  	TMR3ON = 0

//******************************************************************
//*	EUSART BUFFER DEFINITIONS
//******************************************************************
//1,2,4,8,16,32,64,128 or 256 bytes 
#define EUSART_RX_BUFFER_SIZE  64     
#define EUSART_RX_BUFFER_MASK ( EUSART_RX_BUFFER_SIZE - 1 )
// 1,2,4,8,16,32,64,128 or 256 bytes 
#define EUSART_TX_BUFFER_SIZE  8      
#define EUSART_TX_BUFFER_MASK ( EUSART_TX_BUFFER_SIZE - 1 )
unsigned char EUSART_RxBuf[EUSART_RX_BUFFER_SIZE],EUSART_TxBuf[EUSART_TX_BUFFER_SIZE];
unsigned char EUSART_TxHead,EUSART_TxTail,EUSART_RxHead,EUSART_RxTail;

void cleaner(void) //loads receive buffer with 0x00
{
	unsigned int x;
	for(x=0;x<EUSART_RX_BUFFER_SIZE-1;++x)
		EUSART_RxBuf[x] = 0;
}
	
void putch(char c)
{
   while(!TRMT) 	// TRMT is set when TSR is empty
      continue;

  TXREG = c;		// load the register
}
unsigned int recvchar(void)
{
  unsigned char tmptail;
                                // wait for incomming usart_data  
  while ( EUSART_RxHead == EUSART_RxTail );
                                // calculate buffer index  
  tmptail = ( EUSART_RxTail + 1 ) & EUSART_RX_BUFFER_MASK;
  EUSART_RxTail = tmptail;        // store new index  

  return EUSART_RxBuf[tmptail];   // return usart_data  
}

unsigned int sendchar(unsigned int usart_data)
{
  unsigned char tmphead;
                                // calculate buffer index 
  tmphead = ( EUSART_TxHead + 1 ) & EUSART_TX_BUFFER_MASK;
                                // wait for free space in buffer 
  while ( tmphead == EUSART_TxTail );
                                // store usart_data in buffer 
  EUSART_TxBuf[tmphead] = (unsigned char)usart_data;
  EUSART_TxHead = tmphead;        // store new index 

  TXIE = 1;            // enable TX interrupt 

  return usart_data;
}

unsigned char CharInQueue(void)
{
  return(EUSART_RxHead != EUSART_RxTail);
}
//******************************************************************
//*	Init EUSART Function
//******************************************************************
void init_EUSART(void)
{
  SPBRG = 12;	//load baud rate divisor-calc didn't work here	
  TRISC7 = 1;	//receive pin
  TRISC6 = 0;	//transmit pin
  TXSTA = 0x04;	//high speed baud rate	BRGH = 1
  RCSTA = 0x80;	//enable serial port and serial port pins
  
  EUSART_RxTail = 0x00;	//flush receive buffer 
  EUSART_RxHead = 0x00;
  EUSART_TxTail = 0x00;	//flush transmit buffer
  EUSART_TxHead = 0x00;
  RCIP = 1;			//receive interrupt = high priority
  TXIP = 1;			//transmit interrupt = high priority
  RCIE = 1;			//enable receive interrupt
  PEIE = 1;			//enable all unmasked peripheral interrupts
  GIE = 1;			//enable all unmasked interrupts
  CREN = 1;			//enable EUSART1 receiver
  TXIE = 0;			//disable EUSART1 transmit interrupt
  TXEN = 1;			//transmitter enabled
}
//****************************************************************************
//*	INITIALIZE ROUTINE
//****************************************************************************
void init()
{
//****************************************************************************
//*	INITIALIZE CLOCK AND IO PORTS
//****************************************************************************
	OSCCON = 0x60;		//4 MHz internal clock
	PLLEN = 0;			//PLL OFF
	TRISA = 0b11111111;
	TRISB = 0b11111111;
	TRISC = 0b11111111;
//******************************************************************
//*	CONFIGURE A2D AND COMPARATORS
//******************************************************************
	ADCON1 = 0b00001111;	//all digital inputs
	ADON = 0;				//A2D OFF
	CMCON = 0x07;			//comparators OFF
//****************************************************************************
//*	INITIALIZE USART
//****************************************************************************
	init_EUSART();
//******************************************************************
//*	CONFIGURE INTERRUPTS
//******************************************************************
	enable_GLOBALint;
}
void interrupt EUSART(void)
{
//SERVICE THE EUSART
 if(RCIF)
 {
   usart_data = RCREG;                   // read the received usart_data 
                                // calculate buffer index 
   tmphead = ( EUSART_RxHead + 1 ) & EUSART_RX_BUFFER_MASK;
   EUSART_RxHead = tmphead;        // store new index 

   if ( tmphead == EUSART_RxTail )
   {
     // ERROR! Receive buffer overflow 
   }
                                
   EUSART_RxBuf[tmphead] = usart_data;   // store received usart_data in buffer 
  }

  
  if(TRMT)
  {
                                // check if all usart_data is transmitted  
   if ( EUSART_TxHead != EUSART_TxTail )
   {
                                // calculate buffer index  
     tmptail = ( EUSART_TxTail + 1 ) & EUSART_TX_BUFFER_MASK;
     EUSART_TxTail = tmptail;      // store new index  

    TXREG = EUSART_TxBuf[tmptail];  // start transmitiON  
   }
   else
   {
     TXIE = 0;         // disable TX interrupt  
   }
  }
}
		
//******************************************************************
//*	MAIN SERVICE LOOP
//******************************************************************
void main(void)
{
	char i;
	
	init();
	cleaner();
	i = 0;
	printf("AT+N\r\n");
 	for(temp16=0;temp16<1000;++temp16)
 		i= 0;
		do{
		our_buffer[i++] = recvchar();
		}while(CharInQueue());
	while(1);
}